@@ -184,6 +184,11 @@ bool Fragment::OnChanged (bool save)
 	AnalContent (m_StartSel, CurPos);
 	m_bLoading = true;
 	/*main atom management*/
+	if (m_EndSel > CurPos && m_BeginAtom >= m_EndSel) {
+		// if text was deleted before the atom, adjust its bounds
+		m_BeginAtom -= m_EndSel - CurPos;
+		m_EndAtom -= m_EndSel - CurPos;
+	}
 	if (m_buf.length () < m_EndAtom) { // needed if the symbol of part of it has been destroyed
 		m_Atom->SetZ (0);
 		m_EndAtom = m_buf.length ();
@@ -214,8 +219,8 @@ bool Fragment::OnChanged (bool save)
 			i--;
 		}
 		if (r) {
-			CurPos = m_StartSel + strlen (sy);
-			m_BeginAtom = m_StartSel;
+			CurPos = start + strlen (sy);
+			m_BeginAtom = start;
 			m_EndAtom = CurPos;
 			if (residue)
 				residue->SetResidue (r);
