@@ -29,9 +29,11 @@
 #include "reaction-arrow.h"
 #include "reaction-operator.h"
 #include "document.h"
+#include "mechanism-arrow.h"
 #include "theme.h"
 #include "view.h"
 #include "widgetdata.h"
+#include <gccv/item.h>
 
 using namespace gcu;
 using namespace std;
@@ -40,20 +42,20 @@ namespace gcp {
 
 TypeId ReactionStepType = NoType;
 
-ReactionStep::ReactionStep (): Object (ReactionStepType)
+ReactionStep::ReactionStep (): MechanismStep (ReactionStepType)
 {
 	SetId ("rs1");
 	m_bLoading = false;
 }
 
-ReactionStep::ReactionStep (Reaction *reaction, map<double, Object*>& Children, map<Object*, gccv::Rect> Objects): Object (ReactionStepType)
+ReactionStep::ReactionStep (Reaction *reaction, map<double, Object*>& Children, map<Object*, gccv::Rect> Objects): MechanismStep (ReactionStepType)
 {
 	SetId ("rs1");
 	reaction->AddChild (this);
 	GetDocument ()->EmptyTranslationTable ();
 	Document *pDoc = dynamic_cast <Document*> (GetDocument ());
+	View *view = pDoc->GetView ();
 	Theme *pTheme = pDoc->GetTheme ();
-	WidgetData  *pData= (WidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
 	map<double, Object*>::iterator im, endm;
 	double x, y, x0, y0, x1, y1;
 	gccv::Rect *rect;
@@ -71,8 +73,7 @@ ReactionStep::ReactionStep (Reaction *reaction, map<double, Object*>& Children,
 		AddChild (pOp);
 		pOp->SetCoords (x / pTheme->GetZoomFactor (), y);
 		pDoc->AddObject (pOp);
-/*		gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));
-		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (pData->Items[pOp]), &x0, &y0, &x1, &y1);*/
+		dynamic_cast <gccv::ItemClient *> (pOp)->GetItem ()->GetBounds (x0, y0, x1, y1);
 		pOp->Move ((x - x0) / pTheme->GetZoomFactor (), 0);
 		x += pTheme->GetSignPadding () + x1 - x0;
 		cur = (*im).second;
@@ -82,8 +83,7 @@ ReactionStep::ReactionStep (Reaction *reaction, map<double, Object*>& Children,
 		cur->Move ((x - rect->x0) / pTheme->GetZoomFactor (), y - y0);
 		x+= rect->x1 - rect->x0;
 	}
-/*	Update (pData->Canvas);
-	gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));*/
+	view->Update (this);
 	m_bLoading = false;
 }
 
@@ -137,8 +137,8 @@ xmlNodePtr ReactionStep::Save (xmlDocPtr xml) const
 	Object const *obj = GetFirstChild (i);
 	xmlNodePtr child;
 	while (obj) {
-		if ((*i).second->GetType () != ReactionOperatorType) {
-			if ((child = (*i).second->Save (xml)))
+		if (obj->GetType () != ReactionOperatorType) {
+			if ((child = obj->Save (xml)))
 				xmlAddChild (node, child);
 			else
 				return NULL;
@@ -159,18 +159,20 @@ bool ReactionStep::Load (xmlNodePtr node)
 	map<string, Object*>::iterator i;
 	Object *pObj = GetFirstChild (i);
 	Document *pDoc = dynamic_cast <Document*> (GetDocument ());
+	View *view = pDoc->GetView ();
 	Theme *pTheme = pDoc->GetTheme ();
-	WidgetData  *pData= (WidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
+	WidgetData  *pData= view->GetData ();
 	map<double, Object*>::iterator im, endm;
 	double x, y, x0, y0, x1, y1;
-//	gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));
 	while (pObj) {
-		pData->GetObjectBounds (pObj, &rect);
-		x = (rect.x0 + rect.x1) / 2;
-		while (Children[x] != NULL)
-			x += 1e-5;
-		Children[x] = pObj;
-		Objects[pObj] = rect;
+		if (pObj->GetType () != gcp::MechanismArrowType) {
+			pData->GetObjectBounds (pObj, &rect);
+			x = (rect.x0 + rect.x1) / 2;
+			while (Children[x] != NULL)
+				x += 1e-5;
+			Children[x] = pObj;
+			Objects[pObj] = rect;
+		}
 		pObj = GetNextChild (i);
 	}
 	im = Children.begin ();
@@ -185,15 +187,14 @@ bool ReactionStep::Load (xmlNodePtr node)
 		AddChild (pOp);
 		pOp->SetCoords(x / pTheme->GetZoomFactor (), y);
 		pDoc->AddObject(pOp);
-/*		gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));
-		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (pData->Items[pOp]), &x0, &y0, &x1, &y1);*/
+		dynamic_cast <gccv::ItemClient *> (pOp)->GetItem ()->GetBounds (x0, y0, x1, y1);
 		pOp->Move ((x - x0) / pTheme->GetZoomFactor (), 0);
 		x += pTheme->GetSignPadding () + x1 - x0;
 		pObj = (*im).second;
 		rect = Objects[pObj];
 		x+= rect.x1 - rect.x0;
 	}
-//	Update (pData->Canvas);
+	view->Update (this);
 	m_bLoading = false;
 	return true;
 }
@@ -205,7 +206,7 @@ double ReactionStep::GetYAlign ()
 	return ((*i).second)? (*i).second->GetYAlign (): 0.;
 }
 
-bool ReactionStep::OnSignal (SignalId Signal, Object *Child)
+bool ReactionStep::OnSignal (SignalId Signal, G_GNUC_UNUSED Object *Child)
 {
 	if (Signal == OnChangedSignal) {
 		if (m_bLoading)
@@ -222,7 +223,6 @@ bool ReactionStep::OnSignal (SignalId Signal, Object *Child)
 		WidgetData  *pData= (WidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
 		map<double, Object*>::iterator im, endm;
 		double x, y, x0, y0, x1, y1;
-//		gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));
 		while (pObj) {
 			if (pObj->GetType () == ReactionOperatorType)
 				Operators.push_front (pObj);
@@ -254,8 +254,7 @@ bool ReactionStep::OnSignal (SignalId Signal, Object *Child)
 			AddChild (pOp);
 			pOp->SetCoords(x / pTheme->GetZoomFactor (), y);
 			pDoc->AddObject(pOp);
-	/*		gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));
-			gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (pData->Items[pOp]), &x0, &y0, &x1, &y1);*/
+			dynamic_cast <gccv::ItemClient *> (pOp)->GetItem ()->GetBounds (x0, y0, x1, y1);
 			pOp->Move ((x - x0) / pTheme->GetZoomFactor (), 0);
 			x += pTheme->GetSignPadding () + x1 - x0;
 			pObj = (*im).second;
@@ -263,7 +262,7 @@ bool ReactionStep::OnSignal (SignalId Signal, Object *Child)
 			pObj->Move ((x - rect.x0) / pTheme->GetZoomFactor (), y - pObj->GetYAlign ());
 			x+= rect.x1 - rect.x0;
 		}
-	//	Update (pData->Canvas);
+		pDoc->GetView ()->Update (this);
 		return true;
 	} else
 		return true;
@@ -277,12 +276,4 @@ void ReactionStep::RemoveArrow (ReactionArrow *arrow) {
 	}
 }
 
-void ReactionStep::Add (GtkWidget* w) const
-{
-	/* Now that Object::Add adds children, we need to override this method
-	because a molecule children (atoms, bonds, and fragments) are added to
-	the widget by the document. FIXME! FIXME! FIXME! CHANGE THIS OLD WEIRD CODE!
-	*/
-}
-
 }	//	namespace gcp
