@@ -4,7 +4,7 @@
  * GChemPaint libray
  * text-object.h 
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,19 +28,26 @@
 #include <gcu/object.h>
 #include <gcu/macros.h>
 #include <gccv/structs.h>
+#include <gccv/text-client.h>
 #include <gccv/text-tag.h>
 #include <pango/pango-layout.h>
 #include <string>
 
+namespace gccv {
+	class Text;
+}
+
 /*!\file*/
 namespace gcp {
 
+class TextEditor;
+
 /*!\class TextObject gcp/text-object.h
 
 The base class for texts. This class is virtual with one pure virtual method
 (gcp::TextObject::OnSave).
 */
-class TextObject: public gcu::Object
+class TextObject: public gcu::Object, public gccv::TextClient
 {
 public:
 /*!
@@ -88,13 +95,6 @@ must be overloaded in derived classes.
 */
 	virtual bool OnChanged (bool save) = 0;
 /*!
-@param bounds the bounds of the selection.
-
-Called during edition when the selection has changed, that is after any change.
-The signature of this method will change with the deprecation of GnomeCanvas.
-*/
-	void OnSelChanged (gccv::TextSelBounds *bounds);
-/*!
 @param xml the xml document.
 @param node the node representing the text.
 
@@ -141,6 +141,14 @@ supported: GCU_PROP_TEXT_TEXT
 */
 	virtual std::string GetProperty (unsigned property) const;
 
+	// virtual methods for the gccv:TextClient interface
+/*!
+*/
+	void SelectionChanged (unsigned start, unsigned end);
+/*!
+*/
+	void TextChanged (unsigned pos);
+
 protected:
 /*!
 The horizontal of the object in canvas units.
@@ -196,14 +204,16 @@ first line is used for vertical alignment.
 /*!\fn GetAnchor()
 @return the current GtkAnchor.
 */
-GCU_PROT_PROP (GtkAnchorType, Anchor);
+//GCU_PROT_PROP (GtkAnchorType, Anchor)
 /*!\var m_TagList
 The tags list associated with the object.
 */
 /*!\fn GetTagList
 @return a copy of the tags list for this object.
 */
-GCU_PROT_PROP (gccv::TextTagList, TagList);
+GCU_PROT_PROP (gccv::TextTagList, TagList)
+GCU_PROP (TextEditor*, Editor)
+GCU_PROT_PROP (gccv::Text*, TextItem)
 };
 
 }	// namespace gcp
