@@ -64,7 +64,6 @@ Tools::Tools (Application *App):
 	m_Mendeleiev = reinterpret_cast <GtkComboPeriodic *> (w);
 	gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
 	gtk_combo_periodic_set_element (GTK_COMBO_PERIODIC (w), pApp->GetCurZ ());
-	go_combo_box_set_tearable (GO_COMBO_BOX (w), TearableMendeleiev);
 	g_signal_connect_swapped (G_OBJECT (w), "changed", G_CALLBACK (element_changed_cb), this);
 	w = GetWidget ("help-btn");
 	g_signal_connect_swapped (G_OBJECT (w), "clicked", G_CALLBACK (help_cb), this);
@@ -136,10 +135,11 @@ void Tools::RegisterTool (GtkWidget *w)
 {
 	char const *name = gtk_widget_get_name (w);
 	Application *App = dynamic_cast<Application*> (m_App);
-	if (strncmp (name, "Gtk", 3))
+	if (strncmp (name, "Gtk", 3)) {
 		App->SetToolItem (name, w);
-	Tool *tool = App->GetTool (name);
-	m_Pages[tool] = -1;
+		Tool *tool = App->GetTool (name);
+		m_Pages[tool] = -1;
+	}
 }
 
 void Tools::OnElementChanged (int Z)
@@ -159,11 +159,6 @@ void Tools::OnElementChanged (int Z)
 		gtk_label_set_text (GTK_LABEL (icon), Element::Symbol (Z));
 }
 
-void Tools::Update (void)
-{
-	go_combo_box_set_tearable (GO_COMBO_BOX (m_Mendeleiev), TearableMendeleiev);
-}
-
 void Tools::OnHelp ()
 {
 	m_App->OnHelp (m_Tool->GetHelpTag ());
