@@ -182,7 +182,7 @@ Used when saving to get properties from the document.
 Called by the application whe the document has been loaded to update the title
 and add some lines.
 */
-	void Loaded ();
+	bool Loaded () throw (LoaderError);
 /*!
 	@param object the Object instance to add as a child.
 */
@@ -292,15 +292,15 @@ List of the views of the document.
 */
 	std::list <CrystalView *> m_Views;
 
-/*!
-The space group associated with the lattice.
+/*!GetSpaceGroup()
+@return the space group associated with the lattice.
 */
-	SpaceGroup const *m_SpaceGroup;
 
 GCU_RO_PROP (std::string, NameCommon);
 GCU_RO_PROP (std::string, NameSystematic);
 GCU_RO_PROP (std::string, NameMineral);
 GCU_RO_PROP (std::string, NameStructure);
+GCU_RO_PROP (SpaceGroup const *, SpaceGroup);
 };
 
 extern gchar const *LatticeName[];
