@@ -2,9 +2,9 @@
 
 /* 
  * Gnome Chemistry Utils
- * chemistry/document.h 
+ * gcu/document.h 
  *
- * Copyright (C) 2004-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,8 +28,8 @@
 
 #include <gcu/object.h>
 #include <gcu/dialog-owner.h>
-#include <gcu/loader-error.h>
 #include <gcu/macros.h>
+#include <gcu/loader-error.h>
 #include <string>
 #include <set>
 
@@ -42,14 +42,6 @@ class Dialog;
 class Molecule;
 class Residue;
 
-class PendingTarget
-{
-public:
-	Object *parent;
-	Object *owner;
-	Object **target;
-};
-
 /*!\class Document gcu/document.h
 This class is the base document class.
 */
@@ -98,11 +90,11 @@ to avoid errors on the next paste event.
 /*!
 @param title the new document title.
 */
-	void SetTitle (std::string& title) {m_Title = title;}
+	virtual void SetTitle (std::string& title) {m_Title = title;}
 /*!
 @param title the new document title.
 */
-	void SetTitle (char const *title) {m_Title = title;}
+	virtual void SetTitle (char const *title) {m_Title = title;}
 /*!
 @return the current document title.
 */
@@ -136,26 +128,30 @@ implementation just returns NULL.
 @param ambiguous where to store the boolean telling if the symbol is ambiguous
 or NULL.
 
-Documents might own not global residues with the samesymbol or name
+Documents might own not global residues with the same symbol or name
 but a different meaning from the standard residue.
 @return the Residue* found or NULL.
 */
 	virtual Residue const *GetResidue (char const *symbol, bool *ambiguous = NULL);
-
+/*!
+This method is used to get the active window of the document. Derived classes should, override
+it to return a consistent value. Default implementation returns NULL.
+@return the active GtkWindow for the document if any.
+*/
+	virtual GtkWindow *GetGtkWindow () {return NULL;}
 /*!
 @param id the id of the target Object.
 @param target where to store the found object.
 @param parent the ancestor of the search object or NULL.
-@param owner the owner of the reference to the search object.
 
 Search the descendant of \a parent, or of the whole document if \a parent is not set
 whose Id is \id. I not found, the parameters are stored for post loading processing
 using gcu::Document::Loaded(), and \a target is set to NULL.
-if \a owner is not NULL, post processing will call its OnLoaded() method.
 @return true if the target object was found.
 */
-	bool SetTarget (char const *id, Object **target, Object *parent, Object *owner = NULL) throw (std::runtime_error);
+	bool SetTarget (char const *id, Object **target, Object *parent) throw (std::runtime_error);
 
+protected:
 /*!
 Processes pending references resulting from failed calls to SetTarget().
 @return true if any reference was set.
@@ -177,7 +173,7 @@ GetNewId returns the translated id
 
 private:
 	std::map <std::string, std::string> m_TranslationTable;//used when Ids translations are necessary (on pasting...)
-	std::map <std::string, std::list <PendingTarget> > m_PendingTable;//used to set pointers to objects when loading does not occur in the ideal order
+	std::map <std::string, std::pair <Object**, Object*> > m_PendingTable;//used to set pointers to objects when loading does not occur in the ideal order
 
 protected:
 /*!
@@ -205,14 +201,18 @@ Tells if the document is empty or not.
 @return true if the document does not contain anything, false otherwise.
 */
 GCU_PROT_PROP (bool, Empty);
-/*!\var m_Scale
-The scale factor to be used in IO operations. all coordinates should be multiplied by this factor
-when loading and divided when saving. Default value is 1.
+/*!\fn SetScale(double scale)
+@param scale the new scale factor.
+
+Sets the new scale factor.
 */
 /*!\fn GetScale()
 @return the current scale factor.
 */
-GCU_PROT_PROP (double, Scale);
+/*!\fn GetRefScale()
+@return the current scale factor as a reference.
+*/
+GCU_PROP (double, Scale);
 };
 
 
