@@ -352,28 +352,6 @@ Generally, the iteration is initialized by a call to Object::GetNodeByName.
 */
 	xmlNodePtr GetNextNodeByName (xmlNodePtr node, char const *Name);
 /*!
-@param w the GtkWidget inside which the Object will be displayed.
-
-Used to add a representation of the Object in the widget. This method might be overrided for displayable Object classes
-unless the application uses another mechanism.
-*/
-	virtual void Add (GtkWidget* w) const;
-/*!
-@param w the GtkWidget inside which the Object is displayed.
-
-Used to update the representation of the Object in the widget. This method might be overrided for displayable Object classes
-unless the application uses another mechanism.
-*/
-	virtual void Update (GtkWidget* w) const;
-/*!
-@param w the GtkWidget inside which the Object is displayed.
-@param state the selection state of the Object.
-
-Used to set the selection state of the Object inside the widget. The values of state are application dependant and have no
-default value.
-*/
-	virtual void SetSelected (GtkWidget* w, int state);
-/*!
 @return true if the Object has at least a child an false if it has none.
 */
 	bool HasChildren () const {return m_Children.size () != 0;}
@@ -476,6 +454,13 @@ Use this method to iterate through the list of Object instances linked to the ob
 */
 	Object* GetNextLink (std::set<Object*>::iterator& i);
 
+/*!
+@param object the object to link.
+
+Adds a link to \a object.
+*/
+	void Link (Object *object);
+
 /*!
 @param object the object to unlink.
 
@@ -486,7 +471,7 @@ Unlinks object and calls Object::OnUnlink.
 /*!
 @param object the object just unlinked by Object::Unlink.
 
-Virtual method called when an object hs been unlinked. Programs should not call it
+Virtual method called when an object has been unlinked. Programs should not call it
 directly, but should call Object::OnUnlink instead.
 */
 	virtual void OnUnlink (Object *object);
@@ -528,6 +513,11 @@ it  means that the document has been changed.
 */
 	void SetDirty (bool dirty = true);
 
+/*!
+Deletes all children. Derived classes might override this method if needed.
+*/
+	virtual void Clear ();
+
 /*!
 @param TypeName the name of the new type.
 @param CreateFunc a pointer to a function returning a pointer to a new object of the new type.
