@@ -24,17 +24,6 @@
 #include "application.h"
 #include "spectrumdoc.h"
 #include "spectrumview.h"
-#include <goffice/data/go-data-simple.h>
-#include <goffice/graph/gog-axis.h>
-#include <goffice/graph/gog-plot.h>
-#include <goffice/graph/gog-series-lines.h>
-#include <goffice/graph/gog-style.h>
-#include <goffice/math/go-fft.h>
-#include <goffice/math/go-math.h>
-#include <goffice/math/go-rangefunc.h>
-#include <goffice/math/go-regression.h>
-#include <goffice/utils/go-color.h>
-#include <gio/gio.h>
 #include <glib/gi18n-lib.h>
 #include <cstring>
 #include <sstream>
@@ -668,8 +657,8 @@ parse_line:
 		case JCAMP_PEAK_TABLE: {
 			// in that case, add drop lines ans remove the normal line
 			GogSeries *series = m_View->GetSeries ();
-			gog_object_add_by_name (GOG_OBJECT (series), "Vertical drop lines", GOG_OBJECT (g_object_new (GOG_SERIES_LINES_TYPE, NULL)));
-			GogStyle *style = gog_styled_object_get_style (GOG_STYLED_OBJECT (series));
+			gog_object_add_by_name (GOG_OBJECT (series), "Vertical drop lines", GOG_OBJECT (g_object_new (GOG_TYPE_SERIES_LINES, NULL)));
+			GOStyle *style = go_styled_object_get_style (GO_STYLED_OBJECT (series));
 			style->line.dash_type = GO_LINE_NONE;
 			style->line.auto_dash = false;
 		}
@@ -1396,7 +1385,7 @@ out:
 		g_signal_handler_unblock (m_XAxisInvertBtn, m_XAxisInvertSgn);
 	}
 	m_View->SetAxisBounds (GOG_AXIS_X, minx, maxx, invert_axis);
-	m_View->SetAxisBounds (GOG_AXIS_Y, miny, maxy, invert_axis);
+	m_View->SetAxisBounds (GOG_AXIS_Y, miny, maxy, false);
 	if (hide_y_axis)
 		m_View->ShowAxis (GOG_AXIS_Y, false);
 	/* Add axes labels */
@@ -1412,7 +1401,7 @@ void SpectrumDocument::ReadDataLine (char const *data, list<double> &l)
 	char buf[32], c = data[0];
 	double val = 0., newval = 0.;
 	bool pos, diff = false;
-	char *eq = strstr (data, "$$");
+	char *eq = strstr (const_cast <char *> (data), "$$");
 	if (eq)
 		*eq = 0;
 	pos = true;
@@ -1426,7 +1415,7 @@ void SpectrumDocument::ReadDataLine (char const *data, list<double> &l)
 		case '+':
 			c = data[i++];
 			if ((c < '0' || c > '9') && c != '.') // FIXME: throw an exception
-				;
+				{;}
 			continue;
 		case '.':
 		case '0':
@@ -1557,18 +1546,17 @@ void SpectrumDocument::ReadDataLine (char const *data, list<double> &l)
 	}
 }
 
-void SpectrumDocument::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const
+void SpectrumDocument::DoPrint (G_GNUC_UNUSED GtkPrintOperation *print, GtkPrintContext *context, G_GNUC_UNUSED int page) const
 {
 	cairo_t *cr;
 	gdouble width, height;
-	GtkWidget *widget = m_View->GetWidget ();
 
 	cr = gtk_print_context_get_cairo_context (context);
 	width = gtk_print_context_get_width (context);
 	height = gtk_print_context_get_height (context);
 	int w, h; // size in points
-	w = widget->allocation.width;
-	h = widget->allocation.height;
+	w = m_View->GetWidth ();
+	h = m_View->GetHeight ();
 	switch (GetScaleType ()) {
 	case GCU_PRINT_SCALE_NONE:
 		break;
@@ -1967,7 +1955,7 @@ double (*SpectrumDocument::GetConversionFunction (SpectrumUnitType oldu, Spectru
 void SpectrumDocument::OnShowIntegral ()
 {
 	m_IntegralVisible = !m_IntegralVisible;
-	GogStyle *style;
+	GOStyle *style;
 	if (m_IntegralVisible) {
 		if (integral < 0) {
 			integral = variables.size ();
@@ -2037,18 +2025,18 @@ void SpectrumDocument::OnShowIntegral ()
 			gog_series_set_dim (v.Series, 0, godata, NULL);
 			godata = go_data_vector_val_new (v.Values, v.NbValues, NULL);
 			gog_series_set_dim (v.Series, 1, godata, NULL);
-			GogStyledObject *axis = GOG_STYLED_OBJECT (g_object_new (GOG_AXIS_TYPE, "major-tick-labeled", false, NULL));
+			GOStyledObject *axis = GO_STYLED_OBJECT (g_object_new (GOG_TYPE_AXIS, "major-tick-labeled", false, NULL));
 			GogPlot	*plot = gog_series_get_plot (v.Series);
 			GogObject *chart = GOG_OBJECT (gog_object_get_parent (GOG_OBJECT (plot)));
 			gog_object_add_by_name (chart, "Y-Axis", GOG_OBJECT (axis));
 			gog_plot_set_axis (plot, GOG_AXIS (axis));
-			style = gog_styled_object_get_style (axis);
+			style = go_styled_object_get_style (axis);
 			style->line.auto_dash = false;
 			style->line.dash_type = GO_LINE_NONE;
-			style = gog_styled_object_get_style (GOG_STYLED_OBJECT (v.Series));
+			style = go_styled_object_get_style (GO_STYLED_OBJECT (v.Series));
 			style->line.auto_dash = false;
 			style->line.auto_color = false;
-			style->line.color = RGBA_RED;
+			style->line.color = GO_COLOR_RED;
 			variables.push_back (v);
 			delete [] xn[0];
 			delete [] xn[1];
@@ -2057,19 +2045,19 @@ void SpectrumDocument::OnShowIntegral ()
 			delete [] xn[4];
 			delete [] yb;
 		} else
-			style = gog_styled_object_get_style (GOG_STYLED_OBJECT (variables[integral].Series));
+			style = go_styled_object_get_style (GO_STYLED_OBJECT (variables[integral].Series));
 		// show the series
 		style->line.dash_type = GO_LINE_SOLID;
 		gog_object_request_update (GOG_OBJECT (variables[integral].Series));
 	} else {
 		// hide the series
-		style = gog_styled_object_get_style (GOG_STYLED_OBJECT (variables[integral].Series));
+		style = go_styled_object_get_style (GO_STYLED_OBJECT (variables[integral].Series));
 		style->line.dash_type = GO_LINE_NONE;
 		gog_object_request_update (GOG_OBJECT (variables[integral].Series));
 	}
 }
 
-void SpectrumDocument::OnTransformFID (GtkButton *btn)
+void SpectrumDocument::OnTransformFID (G_GNUC_UNUSED GtkButton *btn)
 {
 	double *re = variables[R].Values, *im = variables[I].Values;
 	unsigned n = 2;
