@@ -33,56 +33,13 @@
 #include <gcu/molecule.h>
 #include <gcu/printable.h>
 #include <gcu/print-setup-dlg.h>
+#include <gcu/ui-builder.h>
 #include <gcu/residue.h>
 #include <gcu/value.h>
 #include <glib/gi18n.h>
-#include <glade/glade.h>
-#ifndef GOFFICE_HAS_GLOBAL_HEADER
-#   include <gtk/gtkmain.h>
-#   include <gtk/gtkentry.h>
-#   include <gtk/gtklabel.h>
-#   include <gtk/gtkmessagedialog.h>
-#   include <gtk/gtkuimanager.h>
-#   include <gtk/gtkaboutdialog.h>
-#   include <gtk/gtkstock.h>
-#   include <gtk/gtkbox.h>
-#   include <gtk/gtktreeview.h>
-#   include <gtk/gtkcellrenderertext.h>
-#   include <gtk/gtkclipboard.h>
-#   include <goffice/goffice.h>
-#   include <goffice/app/go-plugin.h>
-#   include <goffice/app/go-plugin-loader-module.h>
-#   include <goffice/data/go-data-simple.h>
-#   include <goffice/gtk/go-graph-widget.h>
-#   include <goffice/gtk/goffice-gtk.h>
-#   include <goffice/graph/gog-axis.h>
-#   include <goffice/graph/gog-data-set.h>
-#   include <goffice/graph/gog-object.h>
-#   include <goffice/graph/gog-plot.h>
-#   include <goffice/graph/gog-series.h>
-#   ifndef GO_TYPE_PLUGIN_LOADER_MODULE
-#      define GO_TYPE_PLUGIN_LOADER_MODULE GO_PLUGIN_LOADER_MODULE_TYPE
-#      include <goffice/graph/gog-style.h>
-#      include <goffice/graph/gog-styled-object.h>
-#      define GOStyle GogStyle
-#      define go_styled_object_get_style gog_styled_object_get_style
-#      define GO_STYLED_OBJECT GOG_STYLED_OBJECT
-#   else
-#      include <goffice/graph/gog-object-xml.h>
-#      include <goffice/utils/go-style.h>
-#      include <goffice/utils/go-styled-object.h>
-#   endif
-#   include <goffice/utils/go-locale.h>
-#   include <goffice/utils/go-image.h>
-#   include <goffice/utils/go-line.h>
-#   include <goffice/utils/go-marker.h>
-#endif
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-output-memory.h>
 #include <gsf/gsf-output-gio.h>
-#ifdef GOFFICE_IS_0_6
-#	include <libgnomevfs/gnome-vfs-init.h>
-#endif
 #include <libxml/tree.h>
 #include <iostream>
 #include <cmath>
@@ -109,12 +66,14 @@ public:
 	GtkListStore *pclist;
 
 	GtkWindow *GetGtkWindow () {return window;}
-	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const;
+	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context, int page) const;
 	void OnSaveAsImage ();
 	bool FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc = NULL);
+	static void OnSize (GChemCalc *calc, GtkAllocation *allocation);
 
 private:
 	void ParseNodes (xmlNodePtr node);
+	int m_GraphWidth, m_GraphHeight;
 GCU_PROP (GtkUIManager *, UIManager)
 };
 
@@ -178,7 +137,7 @@ void GChemCalc::ParseNodes (xmlNodePtr node)
 	}
 }
 
-void GChemCalc::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const
+void GChemCalc::DoPrint (G_GNUC_UNUSED GtkPrintOperation *print, GtkPrintContext *context, G_GNUC_UNUSED int page) const
 {
 	cairo_t *cr;
 	gdouble width, height;
@@ -188,8 +147,8 @@ void GChemCalc::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) con
 	height = gtk_print_context_get_height (context);
 
 	int w, h; // size in points
-	w = graph_widget->allocation.width;
-	h = graph_widget->allocation.height;
+	w = m_GraphWidth;
+	h = m_GraphHeight;
 	switch (GetScaleType ()) {
 	case GCU_PRINT_SCALE_NONE:
 		break;
@@ -233,7 +192,7 @@ void GChemCalc::OnSaveAsImage ()
 	FileChooser (this, true, l, NULL, _("Save as image"), GetImageSizeWidget ());
 }
 
-bool GChemCalc::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc)
+bool GChemCalc::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, G_GNUC_UNUSED GtkWindow *window, G_GNUC_UNUSED Document *pDoc)
 {
 	if(bSave) {
 		GFile *file = g_file_new_for_uri (filename);
@@ -272,44 +231,50 @@ bool GChemCalc::FileProcess (const gchar* filename, const gchar* mime_type, bool
 	return false;
 }
 
+void GChemCalc::OnSize (GChemCalc *calc, GtkAllocation *allocation)
+{
+	calc->m_GraphWidth = allocation->width;
+	calc->m_GraphHeight = allocation->height;
+}
+
 GChemCalc *App;
 
-static void on_quit (GtkWidget *widget, void *data)
+static void on_quit (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED void *data)
 {
 	gtk_widget_destroy (GTK_WIDGET (App->window));
 }
 
-static void on_help (GtkWidget *widget, gpointer data)
+static void on_help (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
 	App->OnHelp ();
 }
 
-static void on_web (GtkWidget *widget, gpointer data)
+static void on_web (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
 	App->OnWeb ();
 }
 
-static void on_mail (GtkWidget *widget, gpointer data)
+static void on_mail (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
 	App->OnMail ();
 }
 
-static void on_live_assistance (GtkWidget *widget, gpointer dataw)
+static void on_live_assistance (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer dataw)
 {
 	App->OnLiveAssistance ();
 }
 
-static void on_bug (GtkWidget *widget, gpointer data)
+static void on_bug (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
 	App->OnBug ();
 }
 
-static void on_about_activate_url (GtkAboutDialog *about, const gchar *url, gpointer data)
+static void on_about_activate_url (G_GNUC_UNUSED GtkAboutDialog *about, const gchar *url, G_GNUC_UNUSED gpointer data)
 {
 	App->OnWeb (url);
 }
 
-static void on_about (GtkWidget *widget, void *data)
+static void on_about (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED void *data)
 {
 	const gchar * authors[] = {"Jean Bréfort", NULL};
 	const gchar * comments = _("GChemCalc is a simple calculator for chemists");
@@ -500,7 +465,7 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 
 static GogGraph *graph = NULL;
 
-static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, gpointer data)
+static void on_get_data (G_GNUC_UNUSED GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, G_GNUC_UNUSED gpointer data)
 {
 	guchar *buffer = NULL;
 	char *format = NULL;
@@ -524,11 +489,7 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 			go_locale_untranslated_booleans ();
 		
 			xout = gsf_xml_out_new (output);
-#ifdef HAVE_GO_DATA_SERIALIZE
 			gog_object_write_xml_sax (GOG_OBJECT (graph), xout, NULL);
-#else
-			gog_object_write_xml_sax (GOG_OBJECT (graph), xout);
-#endif
 			g_object_unref (xout);
 		
 			/* go_setlocale restores bools to locale translation */
@@ -559,13 +520,13 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 		g_object_unref (output);
 		g_free (format);
 		gtk_selection_data_set (selection_data,
-					selection_data->target, 8,
+					gtk_selection_data_get_target (selection_data), 8,
 					(guchar *) buffer, osize);
 		g_free (buffer);
 	}
 }
 
-void on_clear_data(GtkClipboard *clipboard, gpointer data)
+void on_clear_data(G_GNUC_UNUSED GtkClipboard *clipboard, G_GNUC_UNUSED gpointer data)
 {
 	g_object_unref (graph);
 	graph = NULL;
@@ -609,12 +570,12 @@ static void on_save_as_image ()
 	App->OnSaveAsImage ();
 }
 
-static void on_mode (GtkRadioAction *action, GtkRadioAction *current)
+static void on_mode (GtkRadioAction *action, G_GNUC_UNUSED GtkRadioAction *current)
 {
 	App->formula.SetParseMode (static_cast <FormulaParseMode> (gtk_radio_action_get_current_value (action)));
 }
 
-static void on_page (GtkNotebook *book, GtkNotebookPage *p, int page)
+static void on_page (G_GNUC_UNUSED GtkNotebook *book, G_GNUC_UNUSED GtkNotebookPage *p, int page)
 {
 	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (App->GetUIManager (), "/MainMenu/FileMenu/SaveAsImage"), page);
 	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (App->GetUIManager (), "/MainMenu/FileMenu/PageSetup"), page);
@@ -624,7 +585,7 @@ static void on_page (GtkNotebook *book, GtkNotebookPage *p, int page)
 }
 
 static GtkActionEntry entries[] = {
-  { "FileMenu", NULL, N_("_File") },
+  { "FileMenu", NULL, N_("_File"), NULL, NULL, NULL },
 	  { "SaveAsImage", GTK_STOCK_SAVE_AS, N_("Save As _Image..."), "<control>I",
 		  N_("Save the current file as an image"), G_CALLBACK (on_save_as_image) },
 	  { "PageSetup", NULL, N_("Page Set_up..."), NULL,
@@ -635,11 +596,11 @@ static GtkActionEntry entries[] = {
 		  N_("Print the current file"), G_CALLBACK (on_print) },
 	  { "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
 		  N_("Quit GChemCalc"), G_CALLBACK (on_quit) },
-  { "EditMenu", NULL, N_("_Edit") },
+  { "EditMenu", NULL, N_("_Edit"), NULL, NULL, NULL },
 	  { "Copy", GTK_STOCK_COPY, N_("_Copy"), "<control>C",
 		  N_("Copy the selection"), G_CALLBACK (on_copy) },
-  { "ModeMenu", NULL, N_("_Mode") },
-  { "HelpMenu", NULL, N_("_Help") },
+  { "ModeMenu", NULL, N_("_Mode"), NULL, NULL, NULL },
+  { "HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL },
 	  { "Help", GTK_STOCK_HELP, N_("_Contents"), "F1",
 		  N_("View help for the Chemical Calculator"), G_CALLBACK (on_help) },
 	  { "Web", NULL, N_("Gnome Chemistry Utils on the _web"), NULL,
@@ -666,7 +627,7 @@ static GtkRadioActionEntry radios[] = {
 		GCU_FORMULA_PARSE_RESIDUE },
 	{ "Ask", NULL, N_("As_k"), NULL,
 		N_("Ask user for the correct interpretation of ambiguous symbols"),
-		GCU_FORMULA_PARSE_ASK },
+		GCU_FORMULA_PARSE_ASK }
 };
 
 static const char *ui_description =
@@ -701,7 +662,7 @@ static const char *ui_description =
 "  </menubar>"
 "</ui>";
 
-gboolean cb_print_version (const gchar *option_name, const gchar *value, gpointer data, GError **error)
+gboolean cb_print_version (G_GNUC_UNUSED const gchar *option_name, G_GNUC_UNUSED const gchar *value, G_GNUC_UNUSED gpointer data, G_GNUC_UNUSED GError **error)
 {
 	char *version = g_strconcat (_("GChemCalc Calculator version: "), VERSION, NULL);
 	puts (version);
@@ -713,7 +674,7 @@ gboolean cb_print_version (const gchar *option_name, const gchar *value, gpointe
 static GOptionEntry options[] = 
 {
   { "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void*) cb_print_version, "prints GChemCalc version", NULL },
-   { NULL }
+   { NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL }
 };
 
 int main (int argc, char *argv[])
@@ -722,12 +683,6 @@ int main (int argc, char *argv[])
 	GError *error = NULL;
 	textdomain (GETTEXT_PACKAGE);
 	gtk_init (&argc, &argv);
-#ifdef GOFFICE_IS_0_6
-	if (!gnome_vfs_init ()) {
-		printf ("Could not initialize GnomeVFS\n");
-		return 1;
-	}
-#endif
 	if (argc > 1 && argv[1][0] == '-') {
 		context = g_option_context_new (_(" [formula]"));
 		g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
@@ -754,13 +709,13 @@ int main (int argc, char *argv[])
 	/* Initialize plugins manager */
 	go_plugins_init (NULL, NULL, NULL, NULL, TRUE, GO_TYPE_PLUGIN_LOADER_MODULE);
 
-	GladeXML *xml =  glade_xml_new (GLADEDIR"/gchemcalc.glade", "gchemcalc", NULL);
-	App->window = GTK_WINDOW (glade_xml_get_widget (xml, "gchemcalc"));
+	UIBuilder *builder = new UIBuilder (UIDIR"/gchemcalc.ui", GETTEXT_PACKAGE);
+	App->window = GTK_WINDOW (builder->GetRefdWidget ("gchemcalc"));
 	g_signal_connect (GTK_OBJECT (App->window), "destroy",
 		 G_CALLBACK (gtk_main_quit),
 		 NULL);
 	
-	GtkWidget *vbox = glade_xml_get_widget (xml, "vbox1");
+	GtkWidget *vbox = builder->GetWidget ("vbox1");
 	GtkUIManager *ui_manager = gtk_ui_manager_new ();
 	App->SetUIManager (ui_manager);
 	GtkActionGroup *action_group = gtk_action_group_new ("MenuActions");
@@ -778,13 +733,13 @@ int main (int argc, char *argv[])
 	GtkWidget *bar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
 	gtk_box_pack_start (GTK_BOX (vbox), bar, FALSE, FALSE, 0);
 	gtk_box_reorder_child (GTK_BOX (vbox), bar, 0);
-	App->markup = GTK_LABEL (glade_xml_get_widget (xml, "markup"));
-	App->raw = GTK_LABEL (glade_xml_get_widget (xml, "raw"));
-	App->weight = GTK_LABEL (glade_xml_get_widget (xml, "weight"));
+	App->markup = GTK_LABEL (builder->GetWidget ("markup"));
+	App->raw = GTK_LABEL (builder->GetWidget ("raw"));
+	App->weight = GTK_LABEL (builder->GetWidget ("weight"));
 
 	//Add composition list
 	App->pclist = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-	GtkTreeView *tree = GTK_TREE_VIEW (glade_xml_get_widget (xml, "composition"));
+	GtkTreeView *tree = GTK_TREE_VIEW (builder->GetWidget ("composition"));
 	gtk_tree_view_set_model (tree, GTK_TREE_MODEL (App->pclist));
 	g_object_unref (App->pclist);
 	GtkCellRenderer *renderer;
@@ -807,10 +762,11 @@ int main (int argc, char *argv[])
 	gtk_tree_view_append_column (tree, column);
 	
 	// Add isotopic pattern chart
-	App->mono = GTK_LABEL (glade_xml_get_widget (xml, "mono"));
-	App->monomass = GTK_LABEL (glade_xml_get_widget (xml, "monomass"));
-	App->pattern_page = glade_xml_get_widget (xml, "pattern");
+	App->mono = GTK_LABEL (builder->GetWidget ("mono"));
+	App->monomass = GTK_LABEL (builder->GetWidget ("monomass"));
+	App->pattern_page = builder->GetWidget ("pattern");
 	App->graph_widget = go_graph_widget_new (NULL);
+	g_signal_connect_swapped (App->graph_widget, "size-allocate", G_CALLBACK (GChemCalc::OnSize), App);
 	gtk_widget_show (App->graph_widget);
 	gtk_box_pack_end (GTK_BOX (App->pattern_page), App->graph_widget, TRUE, TRUE, 0);
 	App->graph = go_graph_widget_get_graph (GO_GRAPH_WIDGET (App->graph_widget));
@@ -831,7 +787,7 @@ int main (int argc, char *argv[])
 	gog_dataset_set_dim (GOG_DATASET (obj), GOG_AXIS_ELEM_MAX, data, &error);
 
 	gtk_widget_hide (App->pattern_page);
-	GtkWidget *w = glade_xml_get_widget (xml, "entry");
+	GtkWidget *w = builder->GetWidget ("entry");
 	g_signal_connect (GTK_OBJECT (w), "activate",
 		 G_CALLBACK (cb_entry_active),
 		 App->window);
@@ -842,10 +798,12 @@ int main (int argc, char *argv[])
 		cb_entry_active (GTK_ENTRY (w), App->window);
 	}
 
-	w = glade_xml_get_widget (xml, "notebook1");
+	w = builder->GetWidget ("notebook1");
 	g_signal_connect (w, "switch-page", G_CALLBACK (on_page), NULL);
 	on_page (NULL, NULL, 0); // force menus deactivation
 
+	delete builder;
+
 	gtk_main ();
 	if (graph)
 		g_object_unref (graph);
